
#include "kwinutils.h"

#include <QLibrary>
#include <QCoreApplication>
#include <QDebug>
#include <QX11Info>
#include <QMargins>
#include <xcb/xcb.h>
// 为了访问 KWinEffects 的保护成员变量
#define protected public
#include <kwineffects.h>
#undef protected




// KWin全局静态对象
namespace KWin {


// 开启窗口合成时才会有此对象
extern EffectsHandler* effects;

static Effect *getEffect(const QString &name)
{
    if (!effects)
        return nullptr;

    for (auto i : effects->loaded_effects) {
        if (i.first == name)
            return i.second;
    }

    return nullptr;
}


static void CreateDesktop()
{
    if (!effects)
        return ;
    effects->setNumberOfDesktops(effects->numberOfDesktops()+1);
}

static void RemoveDesktop(int d)
{
    if (!effects)
        return ;
    if (d <= 0 || d > effects->numberOfDesktops() || effects->numberOfDesktops() == 1)
        return;
    for (const auto& ew: effects->stackingOrder()) {
        if (ew->isOnAllDesktops())
            continue;

        if (ew->desktop() == d){
            ew->closeWindow();
            continue;
        }

        auto dl = ew->desktops();
        if (dl.size() == 0 || dl[0] < d) continue;


        int newd = dl[0] == 1 ? 1 : dl[0] - 1;
        QVector<uint> desks {(uint)newd};
        effects->windowToDesktops(ew, desks);
    }
    effects->setNumberOfDesktops(effects->numberOfDesktops()-1);
}

static int GetDesktopNumber()
{
    return effects->numberOfDesktops();
}

static void RemoveWindow(WId id)
{
    auto effectwindow = effects->findWindow(id);
    if(effectwindow){
        effectwindow->closeWindow();
    }
}



}

KwinAdaptor::KwinAdaptor(QObject *parent)
    :QDBusAbstractAdaptor(parent)
{
    setAutoRelaySignals(true);
}

KwinAdaptor::~KwinAdaptor()
{

}

void KwinAdaptor::CreateDesktop()
{
    QMetaObject::invokeMethod(parent(),"CreateDesktop");
}

void KwinAdaptor::removeDesktop(int id)
{

    QMetaObject::invokeMethod(parent(),"removeDesktop",Q_ARG(int,id));
}

int KwinAdaptor::GetDesktopNumber()
{
    int out0;
    QMetaObject::invokeMethod(parent(),"GetDesktopNumber",Q_RETURN_ARG(int, out0));
    return out0;
}

void KwinAdaptor::removeWindow(QString id)
{
    ulong tmp = id.toULong();
    QMetaObject::invokeMethod(parent(),"removeWindow",Q_ARG(ulong,tmp));
}


KWinUtils::KWinUtils(QObject *parent)
    : QObject(parent)
{

}

KWinUtils::~KWinUtils()
{

}


void KWinUtils::CreateDesktop()
{
    KWin::CreateDesktop();
}

void KWinUtils::removeDesktop(int id)
{
    KWin::RemoveDesktop(id);
}

int KWinUtils::GetDesktopNumber()
{
    return KWin::GetDesktopNumber();
}

void KWinUtils::removeWindow(ulong id)
{
    KWin::RemoveWindow(id);
}
